# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is a Python binding for Network Security Services (NSS).
#
# The Initial Developer of the Original Code is Red Hat, Inc.
#   (Author: John Dennis <jdennis@redhat.com>) 
# 
# Portions created by the Initial Developer are Copyright (C) 2008,2009
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above.  If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****
"""
============
Introduction
============

This package provides a binding for the Network Security Services
(NSS) library. Because NSS directly uses the Netscape Portable Runtime
(NSPR) the binding also provides support for NSPR. There is an
inherent conflict between NSPR and Python, please see the Issues
section for more detail.

General documentation on NSS can be found here:

http://www.mozilla.org/projects/security/pki/nss

General documentation on NSPR can be found here:

http://developer.mozilla.org/en/docs/NSPR_API_Reference

Please note, the documentation included with this package already
encapsultes most of the information at the above two URL's, but is
specific to the python binding of NSS/NSPR. It is suggested you refer
to the python-nss documentation.

Most of the names and symbols in the NSS/NSPR C API have been kept in
the nss-python binding and should be instantly familar or
recognizable. Python has different naming conventions and the
nss-python binding has adhered to the python naming convensions,
Classes are camel case, otherwise symbols are all lower case with
words seperated by underscores. The constants used by NSS/NSPR in C
API have been imported literally to add the programmer who might be
referring to the Mozilla NSS/NSPR documentation and/or header files or
who is porting an existing C application to python. Minor other
changes have been made in the interest of being "Pythonic".

===============
Getting Started
===============

NSS stores it's certificates and private keys in a security database
unlike OpenSSL which references it's certificates and keys via file
pathnames. This means unless you already have an NSS Certificate
Database (CertDB) the first order of business will be to create
one. When a NSS application initializes itself it will need to specify
the path to the CertDB (see "Things All NSS programs must do").

The CertDB is created and manipulated by the command line utilities
certutil and modutil. Both of these programs are part of the nss-tools
RPM. Documentation for these tools can be found here:
http://www.mozilla.org/projects/security/pki/nss/tools

Here is an example of creating a CertDB and populating it. In the
example the CertDB will be created under the directory "./pki", the CA
will be called "myca", the database password will be "myca", and the
server's hostname will be "myhost.example.com".

1. Create the database::

     certutil -N -d ./pki

   This creates a new database under the directory ./pki

2. Create a root CA certificate::

     certutil -d ./pki -S -s "CN=myca" -n myca -x -t "CTu,C,C" -m 1

   This creates an individual certificate and adds it to the
   certificate database with a subject of "CN=myca", a nickname of
   "myca", trust flags indicating for SSL indicating it can issue
   server certificates (C), can issue client certificates (T), and the
   certificate can be used for authentication and signing (u). For
   email and object signing it's trusted to create server
   certificates. The certificate serial number is set to 1.


3. Create a server certificate and sign it. Our example server will
   use this::

     certutil -d pki -S -c myca -s "CN=myhost.example.com" -n myhost -t "u,u,u" -m 2

   This creates an individual certificate issued by the CA "myca" and
   adds it to the certificate database with a subject of
   "CN=myhost.example.com", a nickname of "myhost". The certificate
   serial number is set to 2.

4. Import public root CA's::

     modutil -add ca_certs -libfile /usr/lib/libnssckbi.so -dbdir ./pki

   This is necessary to verify certificates presented by a SSL server a
   NSS client might connect to. When verifying a certificate the NSS
   library will "walk the certificate chain" back to a root CA which
   must be trusted. This command imports the well known root CA's as a
   PKCS #11 module.


===============================
Things All NSS programs must do
===============================

- Import the NSS/NSPR modules::

    from nss.error import NSPRError
    import nss.io as io
    import nss.nss as nss
    import nss.ssl as ssl

  In the interest of code brevity we drop the leading "nss." from the
  module namespace.

- Initialize NSS and indicate the certficate database (CertDB)::

    certdir = './pki'
    ssl.nssinit(certdir)

- If you are implementing an SSL server call config_secure_server()
  (see ssl_example.py)::

    sock = ssl.SSLSocket()
    sock.config_secure_server(server_cert, priv_key, server_cert_kea)

  **WARNING** you must call config_secure_server() for SSL servers, if
  you do not call it the most likely result will be the NSS library
  will segfault (not pretty).

========
Examples
========

There are example programs in under "examples" in the documentation
directory. On Fedora/RHEL/CentOS systems this will be
/usr/share/doc/python-nss.

The ssl_example.py sample implements both a client and server in one
script. You tell it whether to run as a client (-C) or a server (-S)
when you invoke it. The sample shows many of the NSS/NSPR calls and
fully implements basic non-SSL client/server using NSPR, SSL
client/server using NSS, certificate validation, CertDB operations,
and client authentication using certificates.

To get a list of command line options::

  ssl_example.py --help

Using the above example certificate database server can be run like
this::

  ssl_example.py -S -c ./pki -n myhost

The client can be run like this::

  ssl_example.py -C -c ./pki

======
Issues
======

- The current partitioning of the NSS and NSPR API's into Python
  modules (i.e. the Python namespaces and their symbols) is a first
  cut and may not be ideal. One should be prepared for name changes as
  the binding matures.

- NSPR vs. Python

    An original design goal of NSS was to be portable, however NSS
    required access to many system level functions which can vary
    widely between platforms and OS's. Therefore NSPR was written to
    encapsulate system services such as IO, sockets, threads, timers,
    etc. into a common API to insulate NSS from the underlying
    platform.

    In many respects Python and its collection of packages and modules
    provides the same type of platform independence for applications
    and libraries and provides it's own implementation of IO, sockets,
    threads, timers, etc.

    Unfortunately NSPR's and Python's run time abstractions are not
    the same nor can either be configured to use a different
    underlying abstraction layer.

    Currently the NSS binding utilizes *only* the NSPR abstraction
    layer. One consequence of this is it is not possible to create a
    Python socket and use it as the foundation for any NSS functions
    expecting a socket, or visa versa.

    You **must** use the nss.io module to create and manipulate a
    socket used by NSS. You cannot pass this socket to any Python
    library function expecting a socket. The two are not compatible.

    Here are some reasons for this incompatibility, perhaps in the
    future we can find a solution but the immediate goal of the NSS
    Python binding was to expose NSS through Python, not necessarily
    to solve the larger integration issue of Python run-time and NSPR
    run-time. 

    - NSPR would like to hide the underlying platform socket (in the
      NSPR code this is called "osfd"). There are NSPR API's which
      will operate on osfd's

      - One can base a NSPR socket on an existing osfd via:

        - PR_ImportFile()
        - PR_ImportPipe()
        - PR_ImportTCPSocket()
        - PR_ImportUDPSocket()

      - One can obtain the osfd in use by NSPR, either when the
        osfd was imported or because NSPR created the osfd itself via:

	- PR_FileDesc2NativeHandle();

        But note this function is not meant to be public in the NSPR
        API and is documented as being deprecated and carries an
        explicit warning against it's use.

      Once NSPR gets a hold of an osfd it manipulates it in a manner
      as if it were the only owner of the osfd. Other native code
      (e.g. the CPython socket code) which operates on the fd may run
      afoul of NSPR belief it is the only code in the system operating
      on the fd. For example in CPython the non-blocking flag is
      directly set on the fd and non-blocking behavior is implemented
      by the OS. However, NSPR manages non-blocking behavior
      internally to the NSPR library eschewing direct OS support for
      non-blocking. Thus CPython and NSPR are in direct conflict over
      when and how non-blocking is set on an fd. Examples of this
      problem can be seen in the Python socket.makefile() operation
      which takes the fd belonging to a system socket, dups it, and
      calls fdopen() on the dup'ed fd to return a FILE stream (all
      Python file IO is based on file objects utilizing a FILE
      stream). However, the dup'ed fd does not share the same
      non-blocking flag, NSPR explicitly forces the flag off, Python
      wants to directly manipulate it. Dup'ed fd's share their flags
      thus if Python operates on the dup'ed fd returned by NSPR it's
      going to confuse NSPR. Likewise if one sets non-blocking via
      NSPR then Python won't honor the flag because Python is
      expecting the flag to be set on the fd, not in some other
      location (e.g. internal to NSPR).

    - Python's socket implementation is a very thin layer over the
      Berkely socket API. There is very little abstraction, thus
      Python and Python program expect to manipulate sockets directly
      via their fd's.

    - The error and exception model for Python sockets and SSL is an
      almost direct one-to-one mapping of the Posix and OpenSSL
      errors. But NSS uses NSPR errors, thus Python code which has
      exception handlers for sockets and SSL are expecting a complete
      different set of exceptions.

    - Python's SSL implementation is a very thin layer over the
      OpenSSL API, there is little abstraction. Thus there is a
      sizeable body of Python code which expects the OpenSSL model for
      IO ready and has exception handlers based on OpenSSL.

===
FAQ
===

To be added

"""
__version__ = '0.10'




#!/usr/bin/python

# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is a Python binding for Network Security Services (NSS).
#
# The Initial Developer of the Original Code is Red Hat, Inc.
#   (Author: John Dennis <jdennis@redhat.com>)
#
# Portions created by the Initial Developer are Copyright (C) 2008,2009
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above.  If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

import os
import sys
import getopt
import getpass

from nss.error import NSPRError
import nss.io as io
import nss.nss as nss
import nss.ssl as ssl

# -----------------------------------------------------------------------------

# command line parameters, default them to something reasonable
#certdir = '/etc/httpd/alias'
certdir = '/etc/pki/nssdb'
hostname = 'www.verisign.com'
port = 443

request = '''\
GET /index.html HTTP/1.0

'''
# -----------------------------------------------------------------------------
# Callback Functions
# -----------------------------------------------------------------------------

def handshake_callback(sock):
    print "handshake complete, peer = %s" % (sock.get_peer_name())

def auth_certificate_callback(sock, check_sig, is_server, certdb):
    print "auth_certificate_callback: check_sig=%s is_server=%s" % (check_sig, is_server)
    cert_is_valid = False

    cert = sock.get_peer_certificate()
    pin_args = sock.get_pkcs11_pin_arg()
    if pin_args is None:
        pin_args = ()

    print "cert:\n%s" % cert

    # Define how the cert is being used based upon the is_server flag.  This may
    # seem backwards, but isn't. If we're a server we're trying to validate a
    # client cert. If we're a client we're trying to validate a server cert.
    if is_server:
        intended_usage = nss.certificateUsageSSLClient
    else:
        intended_usage = nss.certificateUsageSSLServer

    try:
        # If the cert fails validation it will raise an exception, the errno attribute
        # will be set to the error code matching the reason why the validation failed
        # and the strerror attribute will contain a string describing the reason.
        approved_usage = cert.verify_now(certdb, check_sig, intended_usage, *pin_args)
    except Exception, e:
        print e.strerror
        cert_is_valid = False
        print "Returning cert_is_valid = %s" % cert_is_valid
        return cert_is_valid

    print "approved_usage = %s" % nss.cert_usage_flags(approved_usage)

    # Is the intended usage a proper subset of the approved usage
    if approved_usage & intended_usage:
        cert_is_valid = True
    else:
        cert_is_valid = False

    # If this is a server, we're finished
    if is_server or not cert_is_valid:
        print "Returning cert_is_valid = %s" % cert_is_valid
        return cert_is_valid

    # Certificate is OK.  Since this is the client side of an SSL
    # connection, we need to verify that the name field in the cert
    # matches the desired hostname.  This is our defense against
    # man-in-the-middle attacks.

    hostname = sock.get_hostname()
    print "verifying socket hostname (%s) matches cert subject (%s)" % (hostname, cert.subject)
    try:
        # If the cert fails validation it will raise an exception
        cert_is_valid = cert.verify_hostname(hostname)
    except Exception, e:
        print e.strerror
        cert_is_valid = False
        print "Returning cert_is_valid = %s" % cert_is_valid
        return cert_is_valid

    print "Returning cert_is_valid = %s" % cert_is_valid
    return cert_is_valid

# -----------------------------------------------------------------------------
# Client Implementation
# -----------------------------------------------------------------------------

def client():
    # Get the IP Address of our server
    net_addr = io.NetworkAddress(hostname, port)
    sock = ssl.SSLSocket()
    # Set client SSL socket options
    sock.set_ssl_option(ssl.SSL_SECURITY, True)
    sock.set_ssl_option(ssl.SSL_HANDSHAKE_AS_CLIENT, True)
    sock.set_hostname(hostname)

    # Provide a callback which notifies us when the SSL handshake is
    # complete
    sock.set_handshake_callback(handshake_callback)

    # Provide a callback to verify the servers certificate
    sock.set_auth_certificate_callback(auth_certificate_callback,
                                       nss.get_default_certdb())

    print "client connecting to: %s" % (net_addr)
    sock.connect(net_addr)

    sock.reset_handshake(False) # FIXME: is this needed

    try:
        # Talk to the server
        n_received = 0
        sock.send(request)
        while True:
            buf = sock.recv(1024)
            n_received += len(buf)
            if not buf:
                print "\nclient lost connection, received %d bytes" % (n_received)
                break
    except Exception, e:
        print e.strerror
        sock.shutdown()
        return

    sock.shutdown()
    return

# -----------------------------------------------------------------------------

usage_str = '''
-d --certdir    certificate directory (default: %(certdir)s)
-h --hostname   host to connect to (default: %(hostname)s)
-p --port       host port (default: %(port)s)
''' % {
       'certdir'             : certdir,
       'hostname'            : hostname,
       'port'                : port,
       }

def usage():
    print usage_str

try:
    opts, args = getopt.getopt(sys.argv[1:], "Hd:h:p:",
                               ["help", "certdir=", "hostname=",
                                "port=",
                                ])
except getopt.GetoptError:
    # print help information and exit:
    usage()
    sys.exit(2)


for o, a in opts:
    if o in ("-d", "--certdir"):
        certdir = a
    if o in ("-h", "--hostname"):
        hostname = a
    if o in ("-p", "--port"):
        port = int(a)
    if o in ("-H", "--help"):
        usage()
        sys.exit()

# Perform basic configuration and setup
try:
    nss.nss_init(certdir)
    ssl.set_domestic_policy()
except Exception, e:
    print >>sys.stderr, e.strerror
    sys.exit(1)

client()




#!/usr/bin/python

# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is a Python binding for Network Security Services (NSS).
#
# The Initial Developer of the Original Code is Red Hat, Inc.
#   (Author: John Dennis <jdennis@redhat.com>)
#
# Portions created by the Initial Developer are Copyright (C) 2008,2009
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above.  If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

import os
import sys
import getopt
import getpass

from nss.error import NSPRError
import nss.io as io
import nss.nss as nss
import nss.ssl as ssl

# -----------------------------------------------------------------------------
NO_CLIENT_CERT             = 0
REQUEST_CLIENT_CERT_ONCE   = 1
REQUIRE_CLIENT_CERT_ONCE   = 2
REQUEST_CLIENT_CERT_ALWAYS = 3
REQUIRE_CLIENT_CERT_ALWAYS = 4

# command line parameters, default them to something reasonable
client = False
server = False
password = 'passwd'
use_ssl = True
client_cert_action = NO_CLIENT_CERT
certdir = 'pki'
hostname = os.uname()[1]
server_nickname = 'test_server'
client_nickname = 'test_user'
port = 1234


# -----------------------------------------------------------------------------
# Utility Functions
# -----------------------------------------------------------------------------


# -----------------------------------------------------------------------------
# Callback Functions
# -----------------------------------------------------------------------------

def password_callback(slot, retry, password):
    if password: return password
    return getpass.getpass("Enter password: ");

def handshake_callback(sock):
    print "handshake complete, peer = %s" % (sock.get_peer_name())

def auth_certificate_callback(sock, check_sig, is_server, certdb):
    print "auth_certificate_callback: check_sig=%s is_server=%s" % (check_sig, is_server)
    cert_is_valid = False

    cert = sock.get_peer_certificate()
    pin_args = sock.get_pkcs11_pin_arg()
    if pin_args is None:
        pin_args = ()

    print "cert:\n%s" % cert

    # Define how the cert is being used based upon the is_server flag.  This may
    # seem backwards, but isn't. If we're a server we're trying to validate a
    # client cert. If we're a client we're trying to validate a server cert.
    if is_server:
        intended_usage = nss.certificateUsageSSLClient
    else:
        intended_usage = nss.certificateUsageSSLServer

    try:
        # If the cert fails validation it will raise an exception, the errno attribute
        # will be set to the error code matching the reason why the validation failed
        # and the strerror attribute will contain a string describing the reason.
        approved_usage = cert.verify_now(certdb, check_sig, intended_usage, *pin_args)
    except Exception, e:
        print e.strerror
        cert_is_valid = False
        print "Returning cert_is_valid = %s" % cert_is_valid
        return cert_is_valid

    print "approved_usage = %s" % ', '.join(nss.cert_usage_flags(approved_usage))

    # Is the intended usage a proper subset of the approved usage
    if approved_usage & intended_usage:
        cert_is_valid = True
    else:
        cert_is_valid = False

    # If this is a server, we're finished
    if is_server or not cert_is_valid:
        print "Returning cert_is_valid = %s" % cert_is_valid
        return cert_is_valid

    # Certificate is OK.  Since this is the client side of an SSL
    # connection, we need to verify that the name field in the cert
    # matches the desired hostname.  This is our defense against
    # man-in-the-middle attacks.

    hostname = sock.get_hostname()
    print "verifying socket hostname (%s) matches cert subject (%s)" % (hostname, cert.subject)
    try:
        # If the cert fails validation it will raise an exception
        cert_is_valid = cert.verify_hostname(hostname)
    except Exception, e:
        print e.strerror
        cert_is_valid = False
        print "Returning cert_is_valid = %s" % cert_is_valid
        return cert_is_valid

    print "Returning cert_is_valid = %s" % cert_is_valid
    return cert_is_valid

def client_auth_data_callback(ca_names, chosen_nickname, password, certdb):
    cert = None
    if chosen_nickname:
        try:
            cert = nss.find_cert_from_nickname(chosen_nickname, password)
            priv_key = nss.find_key_by_any_cert(cert, password)
            print "client cert:\n%s" % cert
            return cert, priv_key
        except NSPRError, e:
            print e
            return False
    else:
        nicknames = nss.get_cert_nicknames(certdb, cert.SEC_CERT_NICKNAMES_USER)
        for nickname in nicknames:
            try:
                cert = nss.find_cert_from_nickname(nickname, password)
                print "client cert:\n%s" % cert
                if cert.check_valid_times():
                    if cert.has_signer_in_ca_names(ca_names):
                        priv_key = nss.find_key_by_any_cert(cert, password)
                        return cert, priv_key
            except NSPRError, e:
                print e
        return False

# -----------------------------------------------------------------------------
# Client Implementation
# -----------------------------------------------------------------------------

def Client():
    # Get the IP Address of our server
    net_addr = io.NetworkAddress(hostname, port)
    if use_ssl:
        sock = ssl.SSLSocket()

        # Set client SSL socket options
        sock.set_ssl_option(ssl.SSL_SECURITY, True)
        sock.set_ssl_option(ssl.SSL_HANDSHAKE_AS_CLIENT, True)
        sock.set_hostname(hostname)
        sock.reset_handshake(False) # FIXME: is this needed

        # Provide a callback which notifies us when the SSL handshake is complete
        sock.set_handshake_callback(handshake_callback)

        # Provide a callback to supply our client certificate info
        sock.set_client_auth_data_callback(client_auth_data_callback, client_nickname,
                                           password, nss.get_default_certdb())

        # Provide a callback to verify the servers certificate
        sock.set_auth_certificate_callback(auth_certificate_callback,
                                           nss.get_default_certdb())
    else:
        sock = io.Socket()

    print "client connecting to: %s" % (net_addr)
    sock.connect(net_addr)

    # Talk to the server
    try:
        sock.send("Hello")
        buf = sock.recv(1024)
        if not buf:
            print "client lost connection"
            return
        print "client received: %s" % (buf)
    except Exception, e:
        print e.strerror
        sock.shutdown()
        return

    # End of (simple) protocol session?
    if buf == 'Goodbye':
        sock.shutdown()
        return

# -----------------------------------------------------------------------------
# Server Implementation
# -----------------------------------------------------------------------------

def Server():
    # Perform basic SSL server configuration
    ssl.set_default_cipher_pref(ssl.SSL_RSA_WITH_NULL_MD5, True)
    ssl.config_server_session_id_cache()

    # Get our certificate and private key
    server_cert = nss.find_cert_from_nickname(server_nickname, password)
    priv_key = nss.find_key_by_any_cert(server_cert, password)
    server_cert_kea = server_cert.find_kea_type();

    print "server cert:\n%s" % server_cert

    # Setup an IP Address to listen on any of our interfaces
    net_addr = io.NetworkAddress(io.PR_IpAddrAny, port)

    if use_ssl:
        sock = ssl.SSLSocket()

        # Set server SSL socket options
        sock.set_pkcs11_pin_arg(password)
        sock.set_ssl_option(ssl.SSL_SECURITY, True)
        sock.set_ssl_option(ssl.SSL_HANDSHAKE_AS_SERVER, True)

        # If we're doing client authentication then set it up
        if client_cert_action >= REQUEST_CLIENT_CERT_ONCE:
            sock.set_ssl_option(ssl.SSL_REQUEST_CERTIFICATE, True)
        if client_cert_action == REQUIRE_CLIENT_CERT_ONCE:
            sock.set_ssl_option(ssl.SSL_REQUIRE_CERTIFICATE, True)
        sock.set_auth_certificate_callback(auth_certificate_callback, nss.get_default_certdb())

        # Configure the server SSL socket
        sock.config_secure_server(server_cert, priv_key, server_cert_kea)
        sock.reset_handshake(True) # FIXME: is this needed?

    else:
        sock = io.Socket()

    # Bind to our network address and listen for clients
    sock.bind(net_addr)
    print "listening on: %s" % (net_addr)
    sock.listen()

    while True:
        # Accept a connection from a client
        client_sock, client_addr = sock.accept()
        if use_ssl:
            client_sock.set_handshake_callback(handshake_callback)

        print "client connect from: %s" % (client_addr)

        while True:
            try:
                # Handle the client connection
                buf = client_sock.recv(1024)
                if not buf:
                    print "server lost lost connection to %s" % (client_addr)
                    break

                print "server received: %s" % (buf)

                client_sock.send("Goodbye")
                client_sock.shutdown(io.PR_SHUTDOWN_RCV)
                break
            except Exception, e:
                print e.strerror
                break
        break

    sock.shutdown()


# -----------------------------------------------------------------------------

usage_str = '''
-C --client     run as the client (default: %(client)s)
-S --server     run as the server (default: %(server)s)
-d --certdir    certificate directory (default: %(certdir)s)
-h --hostname   host to connect to (default: %(hostname)s)
-n --server_nickname server certificate nickname (default: %(server_nickname)s)
-N --client_nickname client certificate nickname (default: %(client_nickname)s)
-w --password   certificate database password (default: %(password)s)
-p --port       host port (default: %(port)s)
-e --encrypt    use SSL (default) (default: %(encrypt)s)
-E --noencrypt  don't use SSL (default: %(noencrypt)s)
-f --require_cert_once (default: %(require_cert_once)s)
-F --require_cert_always (default: %(require_cert_always)s)
-r --request_cert_once (default: %(request_cert_once)s)
-R --request_cert_always (default: %(request_cert_always)s)
-H --help
''' % {
       'client'              : client,
       'server'              : server,
       'certdir'             : certdir,
       'hostname'            : hostname,
       'server_nickname'     : server_nickname,
       'client_nickname'     : client_nickname,
       'password'            : password,
       'port'                : port,
       'encrypt'             : use_ssl is True,
       'noencrypt'           : use_ssl is False,
       'require_cert_once'   : client_cert_action == REQUIRE_CLIENT_CERT_ONCE,
       'require_cert_always' : client_cert_action == REQUIRE_CLIENT_CERT_ALWAYS,
       'request_cert_once'   : client_cert_action == REQUEST_CLIENT_CERT_ONCE,
       'request_cert_always' : client_cert_action == REQUEST_CLIENT_CERT_ALWAYS,
       }

def usage():
    print usage_str

try:
    opts, args = getopt.getopt(sys.argv[1:], "Hd:h:n:N:w:p:CSeEfFrR",
                               ["help", "certdir=", "hostname=",
                                "server_nickname=", "client_nickname=",
                                "password=", "port=",
                                "client", "server", "encrypt", "noencrypt",
                                "require_cert_once", "require_cert_always",
                                "request_cert_once", "request_cert_always",
                                ])
except getopt.GetoptError:
    # print help information and exit:
    usage()
    sys.exit(2)


for o, a in opts:
    if o in ("-d", "--certdir"):
        certdir = a
    elif o in ("-h", "--hostname"):
        hostname = a
    elif o in ("-n", "--server_nickname"):
        server_nickname = a
    elif o in ("-N", "--client_nickname"):
        client_nickname = a
    elif o in ("-w", "--password"):
        password = a
    elif o in ("-p", "--port"):
        port = int(a)
    elif o in ("-C", "--client"):
        client = True
    elif o in ("-S", "--server"):
        server = True
    elif o in ("-e", "--encrypt"):
        use_ssl = True
    elif o in ("-E", "--noencrypt"):
        use_ssl = False
    elif o in ("-f", "--require_cert_once"):
        client_cert_action = REQUIRE_CLIENT_CERT_ONCE
    elif o in ("-F", "--require_cert_always"):
        client_cert_action = REQUIRE_CLIENT_CERT_ALWAYS
    elif o in ("-r", "--request_cert_once"):
        client_cert_action = REQUEST_CLIENT_CERT_ONCE
    elif o in ("-R", "--request_cert_always"):
        client_cert_action = REQUEST_CLIENT_CERT_ALWAYS
    elif o in ("-H", "--help"):
        usage()
        sys.exit()
    else:
        usage()
        sys.exit()

if client and server:
    print "can't be both client and server"
    sys.exit(1)
if not (client or server):
    print "must be one of client or server"
    sys.exit(1)

# Perform basic configuration and setup
if certdir is None:
    nss.nss_init_nodb()
else:
    nss.nss_init(certdir)

ssl.set_domestic_policy()
nss.set_password_callback(password_callback)

# Run as a client or as a server
if client:
    print "starting as client"
    Client()

if server:
    print "starting as server"
    Server()




#!/usr/bin/python

# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is a Python binding for Network Security Services (NSS).
#
# The Initial Developer of the Original Code is Red Hat, Inc.
#   (Author: John Dennis <jdennis@redhat.com>)
#
# Portions created by the Initial Developer are Copyright (C) 2008,2009
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above.  If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

'''
This example will pretty print the contents of a certificate loaded from a
file. This is not the easiest or best way to print a certificate, the nss
module has internal code to do that, all you need to invoke the str()
method of the certificate, or it's format, or format_lines()
method. Something as simple as the following will work:

print cert
print "Certificate is %s" % cert

What this example really aims to do is illustrate how to access the various
components of a cert.
'''

import os
import sys
import getopt
import getpass

from nss.error import NSPRError
import nss.io as io
import nss.nss as nss

# -----------------------------------------------------------------------------
def print_extension(level, extension):
    print nss.indented_format([(level, 'Name: %s' % extension.name),
                               (level, 'Critical: %s' % extension.critical)])

    oid_tag = extension.oid_tag

    if   oid_tag == nss.SEC_OID_PKCS12_KEY_USAGE:
        print nss.indented_format([(level, 'Usages:')])
        print nss.indented_format(nss.make_line_pairs(level+1, nss.x509_key_usage(extension.value)))

    elif oid_tag == nss.SEC_OID_X509_SUBJECT_KEY_ID:
        print nss.indented_format([(level, 'Data:')])
        print nss.indented_format(nss.make_line_pairs(level+1,
              extension.value.der_to_hex(nss.OCTETS_PER_LINE_DEFAULT)))

    elif oid_tag == nss.SEC_OID_X509_CRL_DIST_POINTS:
        pts = nss.CRLDistributionPts(extension.value)
        i = 1
        print nss.indented_format([(level, 'CRL Distribution Points: [%d total]' % len(pts))])
        for pt in pts:
            print nss.indented_format([(level+1, 'Point[%d]:' % i)])
            names = pt.get_general_names()
            print nss.indented_format([(level+2, 'General Names: [%d total]' % len(names))])
            for name in names:
                print nss.indented_format([(level+3, '%s:' % name)])
            print nss.indented_format([(level+2, 'Reasons: %s' % (pt.get_reasons(),))])
            print nss.indented_format([(level+2, 'Issuer: %s' % pt.issuer)])

    elif oid_tag == nss.SEC_OID_X509_AUTH_KEY_ID:
        auth_key_id = nss.AuthKeyID(extension.value)
        print nss.indented_format([(level+1, 'Key ID:')])
        print nss.indented_format(nss.make_line_pairs(level+2,
              auth_key_id.key_id.to_hex(nss.OCTETS_PER_LINE_DEFAULT)))
        print nss.indented_format([(level+1, 'Serial Number: %s' % (auth_key_id.serial_number))])
        print nss.indented_format([(level+1, 'Issuer:' % auth_key_id.get_general_names())])

    elif oid_tag == nss.SEC_OID_X509_BASIC_CONSTRAINTS:
        bc = nss.BasicConstraints(extension.value)
        print nss.indented_format([(level, '%s' % str(bc))])

    elif oid_tag == nss.SEC_OID_X509_EXT_KEY_USAGE:
        print nss.indented_format([(level, 'Usages:')])
        print nss.indented_format(nss.make_line_pairs(level+1, nss.x509_ext_key_usage(extension.value)))

    elif oid_tag in (nss.SEC_OID_X509_SUBJECT_ALT_NAME, nss.SEC_OID_X509_ISSUER_ALT_NAME):
        names = nss.x509_alt_name(extension.value)
        print nss.indented_format([(level+2, 'Alternate Names: [%d total]' % len(names))])
        for name in names:
            print nss.indented_format([(level+3, '%s:' % name)])
       
    print

# -----------------------------------------------------------------------------

usage_str = '''
-p --pem read the certifcate in PEM ascii format (default)
-d --der read the certifcate in DER binary format
'''

def usage():
    print usage_str

try:
    opts, args = getopt.getopt(sys.argv[1:], "hpd",
                               ["help", "pem", "der"])
except getopt.GetoptError:
    # print help information and exit:
    usage()
    sys.exit(2)


filename = 'cert.der'
is_pem_format = True

for o, a in opts:
    if o in ("-H", "--help"):
        usage()
        sys.exit()
    elif o in ("-p", "--pem"):
        is_pem_format = True
    elif o in ("-d", "--der"):
        is_pem_format = False


filename = sys.argv[1]

# Perform basic configuration and setup
nss.nss_init_nodb()

if len(args):
    filename = args[0]

print "certificate filename=%s" % (filename)

# Read the certificate as DER encoded data
si = nss.read_der_from_file(filename, is_pem_format)
# Parse the DER encoded data returning a Certificate object
cert = nss.Certificate(si)

# Get the extension list from the certificate
extensions = cert.extensions

print nss.indented_format([(0, 'Certificate:'),
                           (1, 'Data:')])
print nss.indented_format([(2, 'Version: %d (%#x)' % (cert.version+1, cert.version))])
print nss.indented_format([(2, 'Serial Number: %d (%#x)' % (cert.serial_number, cert.serial_number))])
print nss.indented_format([(2, 'Signature Algorithm: %s' % cert.signature_algorithm)])
print nss.indented_format([(2, 'Issuer: "%s"' % cert.issuer)])
print nss.indented_format([(2, 'Validity:'),
                           (3, 'Not Before: %s' % cert.valid_not_before_str),
                           (3, 'Not After:  %s' % cert.valid_not_after_str)])
print nss.indented_format([(2, 'Subject: "%s"' % cert.subject)])
print nss.indented_format([(2, 'Subject Public Key Info:')])
print nss.indented_format(cert.subject_public_key_info.format_lines(3))

if len(extensions) > 0:
    print nss.indented_format([(1, 'Signed Extensions: (%d)' % len(extensions))])
    for extension in extensions:
        print_extension(2, extension)

print nss.indented_format(cert.signed_data.format_lines(1))

print nss.indented_format([(1, 'Fingerprint (MD5):')])
print nss.indented_format(nss.make_line_pairs(2,
                                              nss.data_to_hex(nss.md5_digest(cert.der_data),
                                              nss.OCTETS_PER_LINE_DEFAULT)))

print nss.indented_format([(1, 'Fingerprint (SHA1):')])
print nss.indented_format(nss.make_line_pairs(2,
                                              nss.data_to_hex(nss.sha1_digest(cert.der_data),
                                              nss.OCTETS_PER_LINE_DEFAULT)))




#!/usr/bin/python

import subprocess
import sys
import os
import getopt
import nss.nss as nss

verbose = False

tests = [
    {'name'            : 'md5',
     'ref_cmd'         : 'md5sum',
     'nss_digest_func' : nss.md5_digest,
     'hash_oid'        : nss.SEC_OID_MD5},
    {'name'            : 'sha1',
     'ref_cmd'         : 'sha1sum',
     'nss_digest_func' : nss.sha1_digest,
     'hash_oid'        : nss.SEC_OID_SHA1},
    {'name'            : 'sha256',
     'ref_cmd'         : 'sha256sum',
     'nss_digest_func' : nss.sha256_digest,
     'hash_oid'        : nss.SEC_OID_SHA256},
    {'name'            : 'sha512',
     'ref_cmd'         : 'sha512sum',
     'nss_digest_func' : nss.sha512_digest,
     'hash_oid'        : nss.SEC_OID_SHA512},
]


def do_test(name, ref_cmd, nss_digest_func, hash_oid, in_filename, chunk_size):
    result = 0

    hash_oid_name = nss.oid_str(hash_oid)

    if verbose:
        print 'running test %s: nss_digest_func=%s hash_oid=%s in_filename=%s' % \
            (name, nss_digest_func.__name__, hash_oid_name, in_filename)

    # read the data in from the file
    ref_data = open(in_filename).read()

    # Run the system hash function to get a reference result.
    # Since we're testing the python-nss binding we assume
    # the system command is entirely independent and correct.
    #
    # Because our digest routines return raw data (e.g. a buffer of octets)
    # and the system hash command returns a hex string which we need to compare agains,
    # and because we sometimes want to print the result of our digest functions
    # always convert our results to a hex string via nss.data_to_hex()
    proc = subprocess.Popen([ref_cmd, in_filename], stdout=subprocess.PIPE)
    status = proc.wait();
    reference_digest = proc.stdout.read().split()[0]
    if verbose:
        print 'reference_digest\n%s' % (reference_digest)

    # Run the test with convenience digest function (e.g. nss.sha256_digest, etc.).
    test_digest =  nss.data_to_hex(nss_digest_func(ref_data), separator=None)
    if verbose: print 'nss %s\n%s' % (nss_digest_func.__name__, test_digest)

    if test_digest != reference_digest:
        result += 1
        print 'nss %s test failed' % (nss_digest_func.__name__)
        print 'reference = %s' % (reference_digest)
        print 'test      = %s' % (test_digest)

    # Run the test using the generic hash_buf function specifying the hash algorithm.
    test_digest =  nss.data_to_hex(nss.hash_buf(hash_oid, ref_data), separator=None)
    if verbose: print 'nss.hash_buf %s\n%s' % (hash_oid_name, test_digest)

    if test_digest != reference_digest:
        result += 1
        print 'nss.hash_buf %s test failed' % (hash_oid_name)
        print 'reference = %s' % (reference_digest)
        print 'test      = %s' % (test_digest)

    # Run the test using the lowest level hashing functions by specifying the hash algorithm.
    # The entire input data is supplied all at once in a single call.
    context = nss.create_digest_context(hash_oid)
    context.digest_begin()
    context.digest_op(ref_data)
    test_digest = nss.data_to_hex(context.digest_final(), separator=None)
    if verbose: print 'nss.digest_context %s\n%s' % (hash_oid_name, test_digest)

    if test_digest != reference_digest:
        result += 1
        print 'nss.digest_context %s test failed' % (hash_oid_name)
        print 'reference = %s' % (reference_digest)
        print 'test      = %s' % (test_digest)

    # Run the test using the lowest level hashing functions by specifying the hash algorithm
    # and feeding 'chunks' of data one at a time to be consumed.
    in_file = open(in_filename, 'r')
    context = nss.create_digest_context(hash_oid)
    context.digest_begin()
    while True:
        in_data = in_file.read(chunk_size)
        if len(in_data) == 0:
            break
        context.digest_op(in_data)

    test_digest = nss.data_to_hex(context.digest_final(), separator=None)
    if verbose: print 'chunked nss.digest_context %s\n%s' % (hash_oid_name, test_digest)

    if test_digest != reference_digest:
        result += 1
        print 'chunked nss.digest_context %s test failed' % (hash_oid_name)
        print 'reference = %s' % (reference_digest)
        print 'test      = %s' % (test_digest)

    return result

def usage():
    print '''\
digest_test [-v -h] filename
    -v --verbose         turn on verbose output
    -h --help            print usage
    -s --size            number of octets processed in one iteration
    -i --infile filename file to be used as test data
'''

def main():
    global verbose
    in_filename = sys.argv[0]
    chunk_size = 128

    try:
        opts, args = getopt.getopt(sys.argv[1:], 'hvs:i:',
                                   ['help', 'verbose', 'size=', 'infile='])
    except getopt.GetoptError, err:
        print str(err) # will print something like 'option -a not recognized'
        usage()
        sys.exit(2)
    verbose = False
    for o, a in opts:
        if o in ('-s', '--size'):
            chunk_size = int(a)
        elif o in ('-i', '--infile'):
            in_filename = a
        elif o in ('-v', '--verbose'):
            verbose = True
        elif o in ('-h', '--help'):
            usage()
            sys.exit(0)
        else:
            assert False, 'unhandled option'

    nss.nss_init_nodb()

    result = 0
    for test in tests:
        result += do_test(test['name'], test['ref_cmd'], test['nss_digest_func'],
                          test['hash_oid'], in_filename, chunk_size)

    if result == 0:
        print 'SUCCESS'
    else:
        print 'FAILED %d tests' % (result)

    sys.exit(result)


if __name__ == '__main__':
    main()



#!/usr/bin/python

import sys
import os
import getopt
import nss.nss as nss

verbose = 0

def setup_contexts(mechanism, key, iv):
    # Get a PK11 slot based on the cipher
    slot = nss.get_best_slot(mechanism)

    # If key was supplied use it, otherwise generate one
    if key:
        if verbose:
            print "using supplied key data"
            print "key:\n%s" % (key)
        key_si = nss.SecItem(nss.read_hex(key))
        sym_key = nss.import_sym_key(slot, mechanism, nss.PK11_OriginUnwrap,
                                     nss.CKA_ENCRYPT, key_si)
    else:
        if verbose:
            print "generating key data"
        sym_key = slot.key_gen(mechanism, None, slot.get_best_key_length(mechanism))

    # If initialization vector was supplied use it, otherwise set it to None
    if iv:
        if verbose:
            print "supplied iv:\n%s" % (iv)
        iv_data = nss.read_hex(iv)
        iv_si = nss.SecItem(iv_data)
        iv_param = nss.param_from_iv(mechanism, iv_si)
    else:
        iv_length = nss.get_iv_length(mechanism)
        if iv_length > 0:
            iv_data = nss.generate_random(iv_length)
            iv_si = nss.SecItem(iv_data)
            iv_param = nss.param_from_iv(mechanism, iv_si)
            if verbose:
                print "generated %d byte initialization vector: %s" % \
                    (iv_length, nss.data_to_hex(iv_data, separator=":"))
        else:
            iv_param = None

    # Create an encoding context
    encoding_ctx = nss.create_context_by_sym_key(mechanism, nss.CKA_ENCRYPT,
                                                 sym_key, iv_param)

    # Create a decoding context
    decoding_ctx = nss.create_context_by_sym_key(mechanism, nss.CKA_DECRYPT,
                                                 sym_key, iv_param)

    return encoding_ctx, decoding_ctx

def simple_test(encoding_ctx, decoding_ctx, plain_text):
    result = 0

    if verbose:
        print "Plain Text:\n%s" % (plain_text)

    # Encode the plain text by feeding it to cipher_op getting cipher text back.
    # Append the final bit of cipher text by calling digest_final
    cipher_text = encoding_ctx.cipher_op(plain_text)
    cipher_text += encoding_ctx.digest_final()

    if verbose:
        print "Cipher Text:\n%s" % (nss.data_to_hex(cipher_text, separator=":"))

    # Decode the cipher text by feeding it to cipher_op getting plain text back.
    # Append the final bit of plain text by calling digest_final
    decoded_text = decoding_ctx.cipher_op(cipher_text)
    decoded_text += decoding_ctx.digest_final()

    if verbose:
        print "Decoded Text:\n%s" % (decoded_text)

    # Validate the encryption/decryption by comparing the decoded text with
    # the original plain text, they should match.
    if decoded_text != plain_text:
        result = 1
        print "FAILED! decoded_text != plain_text"

    if cipher_text == plain_text:
        result = 1
        print "FAILED! cipher_text == plain_text"

    return result

def file_test(encoding_ctx, decoding_ctx, in_filename, chunk_size):
    result = 0

    encrypted_filename = os.path.basename(in_filename) + ".encrypted"
    decrypted_filename = os.path.basename(in_filename) + ".decrypted"

    in_file = open(in_filename, "r")
    encrypted_file = open(encrypted_filename, "w")

    if verbose:
        print "Encrypting file \"%s\" to \"%s\"" % (in_filename, encrypted_filename)

    # Encode the data read from a file in chunks
    while True:
        # Read a chunk of data until EOF, encrypt it and write the encrypted data
        in_data = in_file.read(chunk_size)
        if len(in_data) == 0:   # EOF
            break
        encrypted_data = encoding_ctx.cipher_op(in_data)
        encrypted_file.write(encrypted_data)
    # Done encoding the input, get the final encoded data, write it, close files
    encrypted_data = encoding_ctx.digest_final()
    encrypted_file.write(encrypted_data)
    in_file.close()
    encrypted_file.close()

    # Decode the encoded file in a similar fashion
    if verbose:
        print "Decrypting file \"%s\" to \"%s\"" % (encrypted_filename, decrypted_filename)

    encrypted_file = open(encrypted_filename, "r")
    decrypted_file = open(decrypted_filename, "w")
    while True:
        # Read a chunk of data until EOF, encrypt it and write the encrypted data
        in_data = encrypted_file.read(chunk_size)
        if len(in_data) == 0:   # EOF
            break
        decrypted_data = decoding_ctx.cipher_op(in_data)
        decrypted_file.write(decrypted_data)
    # Done encoding the input, get the final encoded data, write it, close files
    decrypted_data = decoding_ctx.digest_final()
    decrypted_file.write(decrypted_data)
    encrypted_file.close()
    decrypted_file.close()

    # Validate the encryption/decryption by comparing the decoded text with
    # the original plain text, they should match.
    in_data        = open(in_filename).read()
    encrypted_data = open(encrypted_filename).read()
    decrypted_data = open(decrypted_filename).read()
    if decrypted_data != in_data:
        result = 1
        print "FAILED! decrypted_data != in_data"

    if encrypted_data == in_data:
        result = 1
        print "FAILED! encrypted_data == in_data"

    # clean up
    os.unlink(encrypted_filename)
    os.unlink(decrypted_filename)

    return result

def usage():
    print '''\
digest_test [-v -h] filename
    filename     file to be used as test data
    -v --verbose turn on verbose output
    -h --help    print usage
    -s --size    number of octets processed in one iteration
    -m --mech    encryption mechanism name (e.g. CKM_*)
                 name is case insensitive, CKM_ prefix is optional
    -t --text    plain text
    -k --key     key (in hexadecimal format)
    -i --iv      initialization vector (in hexadecimal format)
'''

def main():
    global verbose
    mechanism = nss.CKM_DES_CBC_PAD
    plain_text = "Encrypt me!"
    key = "e8:a7:7c:e2:05:63:6a:31"
    iv = "e4:bb:3b:d3:c3:71:2e:58"
    in_filename = None
    chunk_size = 128

    try:
        opts, args = getopt.getopt(sys.argv[1:], "hvs:m:t:k:i:",
                                   ["help", "verbose", "size=", "mechanism=", "text=",
                                    "key=", "iv="])
    except getopt.GetoptError, err:
        print str(err) # will print something like "option -a not recognized"
        usage()
        sys.exit(2)
    verbose = False
    for o, a in opts:
        if o in ("-s", "--size"):
            chunk_size = int(a)
        elif o in ("-m", "--mech"):
            try:
                mechanism = nss.key_mechanism_type_from_name(a)
            except Exception, e:
                print "error with mech argument (%s)" % (e)
                sys.exit(2)
        elif o in ("-t", "--text"):
            plain_text = a
        elif o in ("-k", "--key"):
            key = a
        elif o in ("-i", "--iv"):
            iv = a
        elif o in ("-v", "--verbose"):
            verbose += 1
        elif o in ("-h", "--help"):
            usage()
            sys.exit(0)
        else:
            assert False, "unhandled option"

    if (len(args) > 1):
        print "expected single file name"
        usage()
        sys.exit(2)
    elif (len(args) == 1):
        in_filename = args[0]

    nss.nss_init_nodb()

    result = 0
    encoding_ctx, decoding_ctx = setup_contexts(mechanism, key, iv)
    result += simple_test(encoding_ctx, decoding_ctx, plain_text)
    if in_filename:
        # In theory we should be able to reuse a context by calling finalize()
        # on it, however at the time of this writing it only works for
        # digest contexts, not encryption/decryption contexts
        # so as a workaround we just create the contexts again
        #
        #encoding_ctx.finalize()
        #decoding_ctx.finalize()
        encoding_ctx, decoding_ctx = setup_contexts(mechanism, key, iv)
        result += file_test(encoding_ctx, decoding_ctx, in_filename, chunk_size)

    if result == 0:
        print "SUCCESS"
    else:
        print "FAILED %d tests" % (result)

    sys.exit(result)


if __name__ == "__main__":
    main()
